import functools

# Safe import of OrderedDict
try:
    from collections import OrderedDict
except ImportError:
    from ordered_dict_26 import OrderedDict

# Safe import of Matplotlib (not required)
try:
    import matplotlib as mpl
except ImportError:
    mpl = None

#################################


# Casts given string value, if possible, to an int, float or returns unchanged
def cast_int_float_string(value):

    try:
        return int(value)

    except ValueError:

        try:
            return float(value)

        except ValueError:
            return value


# Returns true if value is array-like, false otherwise
def is_array_like(value):

    if hasattr(value, '__len__') and (not isinstance(value, str)):
        return True

    return False


# Returns either text of xml.find(key), or None
def find_xml_element(xml, key):

    find_result = xml.find(key)

    if xml.find(key) is not None:
        return find_result.text

    return None


# Takes an XML ElementTree element and creates an equivalent OrderedDict. If cast_values, then text values
# are cast via cast_int_float_string() before storing; if skip_attributes then attributes will not be added;
# tag_modify is a 2-valued tuple, or a tuple of 2-valued tuples; any match in element tags for tag_modify[0]
# is replaced with tag_modify[1].
# XML to dict approach was adapted from http://stackoverflow.com/a/10076823
def xml_to_dict(xml_element, cast_values=False, skip_attributes=False, tag_modify=()):

    # Modify tags if requested
    element_tag = xml_element.tag
    if tag_modify:

        if not is_array_like(tag_modify[0]):
            tag_modify = (tag_modify, )

        for tag in tag_modify:
            element_tag = element_tag.replace(tag[0], tag[1])

    d = {element_tag: OrderedDict() if xml_element.attrib else None}
    children = list(xml_element)

    # Add children
    if children:

        dd = OrderedDict()

        xml_to_dict_func = functools.partial(xml_to_dict, cast_values=cast_values,
                                             skip_attributes=skip_attributes, tag_modify=tag_modify)

        for dc in map(xml_to_dict_func, children):
            for k, v in dc.iteritems():
                try:
                    dd[k].append(v)
                except KeyError:
                    dd[k] = [v]

        ddd = OrderedDict()

        for k, v in dd.iteritems():

            if len(v) == 1:
                ddd[k] = v[0]
            else:
                ddd[k] = v

        d = {element_tag: ddd}

    has_attribs = xml_element.attrib and not skip_attributes

    # Add attributes
    if has_attribs:
        d[element_tag].update((k, v) for k, v in xml_element.attrib.iteritems())

    # Add text elements
    if xml_element.text:
        text = xml_element.text.strip()

        if cast_values:
            text = cast_int_float_string(text)

        if children or has_attribs:
            if text and skip_attributes:
                d[element_tag]['_text'] = text

        else:
            d[element_tag] = text

    return d